@@ -90,7 +90,7 @@ ERROR_INTERNAL = 99
 # If you add new codes, also add the appropriate description
 # to g.py for exception description strings.
 # Thank you, The Management
- 
+
 
 # Event and status codes
 # These are used for the 'status-code' returned by DeviceQuery (STATUS_*)
@@ -280,6 +280,8 @@ EVENT_FAX_MAX = 8999
 
 EVENT_CUPS_QUEUES_CHANGED = 9000 # sent by hp-setup if queues added (or removed)
 
+EVENT_MAX_EVENT = 9999
+
 # end of events
 
 
@@ -425,7 +427,7 @@ STATUS_TO_ERROR_STATE_MAP = {
     EVENT_PCARD_JOB_FAIL : ERROR_STATE_ERROR,
     EVENT_PCARD_UNABLE_TO_MOUNT : ERROR_STATE_ERROR,
     EVENT_PCARD_FILES_TRANSFERED : ERROR_STATE_OK,
-    
+
     # Low supplies
     STATUS_PRINTER_LOW_BLACK_INK : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_LOW_TRI_COLOR_INK : ERROR_STATE_LOW_SUPPLIES,
@@ -438,13 +440,13 @@ STATUS_TO_ERROR_STATE_MAP = {
     STATUS_PRINTER_LOW_PHOTO_YELLOW_INK : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_LOW_PHOTO_GRAY_INK : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_LOW_PHOTO_BLUE_INK : ERROR_STATE_LOW_SUPPLIES,
-    
+
     STATUS_PRINTER_LOW_BLACK_TONER : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_LOW_CYAN_TONER : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_LOW_MAGENTA_TONER : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_LOW_YELLOW_TONER : ERROR_STATE_LOW_SUPPLIES,
     # end
-    
+
 }
 
 
@@ -578,12 +580,15 @@ STATUS_TYPE_LJ = 3
 #STATUS_TYPE_S_SNMP = 5 # DEPRECATED
 STATUS_TYPE_LJ_XML = 6
 #STATUS_TYPE_S_LIDIL = 7 # DEPRECATED
+STATUS_TYPE_PJL = 8
+STATUS_TYPE_PML_AND_PJL = 9 # Same as types 3(tbx)+8(hp:)
 
-# status-battery-check-types
+# status-battery-check
 STATUS_BATTERY_CHECK_NONE = 0
-STATUS_BATTERY_CHECK_STD = 1 # Deskjet 450/460
+STATUS_BATTERY_CHECK_STD = 1 # Deskjet 450/460 (PML or dyn. counters)
+STATUS_BATTERY_CHECK_PML = 2 # Officejet H470 (PML)
 
-# status-dynamic-counters-type
+# status-dynamic-counters
 STATUS_DYNAMIC_COUNTERS_NONE = 0
 STATUS_DYNAMIC_COUNTERS_PCL = 1
 STATUS_DYNAMIC_COUNTERS_PML_SNMP = 2
@@ -607,6 +612,7 @@ FAX_TYPE_NONE = 0
 FAX_TYPE_BLACK_SEND_EARLY_OPEN = 1 # newer models
 FAX_TYPE_BLACK_SEND_LATE_OPEN = 2 # older models
 FAX_TYPE_BLACK_AND_COLOR_SEND = 3 # future
+FAX_TYPE_SOAP = 4
 
 # pcard-types
 PCARD_TYPE_NONE = 0
@@ -690,8 +696,31 @@ IO_MODE_DOT4_BRIDGE = 5
 IO_MODE_MLC_GUSHER = 6
 IO_MODE_MLC_MISER = 7
 
+# plugins
+PLUGIN_NONE = 0
+PLUGIN_REQUIRED = 1
+PLUGIN_OPTIONAL = 2
 
+# embedded web server
+EWS_NONE = 0
+EWS_PRESENT = 1
 
+# panel check
+PANEL_CHECK_TYPE_NONE = 0
+PANEL_CHECK_TYPE_CHECK = 1
 
+# pq diag
+PQ_DIAG_TYPE_NONE = 0
+PQ_DIAG_TYPE_1 = 1
+PQ_DIAG_TYPE_2 = 2
 
+# line feed cal
+LINEFEED_CAL_TYPE_NONE = 0
+LINEFEED_CAL_TYPE_OJ_K550 = 1
+LINEFEED_CAL_TYPE_OJ_PRO_L7XXX = 2
+
+# power-settings
+POWER_SETTINGS_NONE = 0
+POWER_SETTINGS_EPML = 1 # DJ450/DJ460
+POWER_SETTINGS_PML = 2 # OJ H470
 
