@@ -49,8 +49,8 @@ except ImportError:
 DEFAULT_PROBE_BUS = 'usb,par,cups'
 VALID_BUSES = ('par', 'net', 'cups', 'usb') #, 'bt', 'fw')
 VALID_BUSES_WO_CUPS = ('par', 'net', 'usb')
-DEFAULT_FILTER = 'none'
-VALID_FILTERS = ('none', 'print', 'scan', 'fax', 'pcard', 'copy')
+DEFAULT_FILTER = None
+VALID_FILTERS = ('print', 'scan', 'fax', 'pcard', 'copy')
 
 pat_deviceuri = re.compile(r"""(.*):/(.*?)/(\S*?)\?(?:serial=(\S*)|device=(\S*)|ip=(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}[^&]*))(?:&port=(\d))?""", re.IGNORECASE)
 http_pat_url = re.compile(r"""/(.*?)/(\S*?)\?(?:serial=(\S*)|device=(\S*))&loc=(\S*)""", re.IGNORECASE)
@@ -65,10 +65,15 @@ MAX_BUFFER = 8192
 # Cache for model data
 model_dat = models.ModelData()
 
+ip_pat = re.compile(r"""\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b""", re.IGNORECASE)
+
+dev_pat = re.compile(r"""/dev/.+""", re.IGNORECASE)
+
+usb_pat = re.compile(r"""(\d+):(\d+)""", re.IGNORECASE)
+    
+    
 def makeURI(param, port=1):  
-    ip_pat = re.compile(r"""\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b""", re.IGNORECASE)
-    dev_pat = re.compile(r"""/dev/.+""", re.IGNORECASE)
-    usb_pat = re.compile(r"""(\d+):(\d+)""", re.IGNORECASE)
+    
 
     cups_uri, sane_uri, fax_uri = '', '', ''
     found = False
@@ -83,9 +88,13 @@ def makeURI(param, port=1):
         log.debug("Trying parallel with %s" % param)
 
         result_code, uri = hpmudext.make_par_uri(param)
-        if result_code == hpmudext.HPMUD_R_OK: 
+        
+        if result_code == hpmudext.HPMUD_R_OK and uri: 
+            log.debug("Found: %s" % uri)
             found = True
             cups_uri = uri
+        else:
+            log.debug("Not found.")
 
     elif usb_pat.search(param) is not None: # USB
         match_obj = usb_pat.search(param)
@@ -95,18 +104,24 @@ def makeURI(param, port=1):
         log.debug("Trying USB with bus=%s dev=%s..." % (usb_bus_id, usb_dev_id))
         result_code, uri = hpmudext.make_usb_uri(usb_bus_id, usb_dev_id)
 
-        if result_code == ERROR_SUCCESS:
+        if result_code == ERROR_SUCCESS and uri:
+            log.debug("Found: %s" % uri)
             found = True
             cups_uri = uri
+        else:
+            log.debug("Not found.")
 
     elif ip_pat.search(param) is not None: # IPv4 dotted quad
         log.debug("Trying IP address %s" % param)
 
         result_code, uri = hpmudext.make_net_uri(param, port)
 
-        if result_code == hpmudext.HPMUD_R_OK:
+        if result_code == hpmudext.HPMUD_R_OK and uri:
+            log.debug("Found: %s" % uri)
             found = True
             cups_uri = uri
+        else:
+            log.debug("Not found.")
 
     else: # serial
         log.debug("Trying serial number %s" % param)
@@ -131,12 +146,13 @@ def makeURI(param, port=1):
                     hpmudext.close_device(device_id)
 
             if serial.lower() == param.lower():
-                log.debug("Match")
+                #log.debug("Match")
+                log.debug("Found: %s" % d)
                 found = True
                 cups_uri = d
                 break
             else:
-                log.debug("No match")
+                log.debug("Not found.")
 
     if found:
         try:
@@ -174,8 +190,17 @@ def queryModelByURI(device_uri):
     else:
         return queryModelByModel(model)
 
+        
+def __checkFilter(filter, mq):
+    for f, p in filter.items():
+        if f is not None:
+            op, val = p
+            if not op(mq[f], val):
+                return False
+     
+    return True   
 
-def getInteractiveDeviceURI(bus='cups,usb,par', filter='none', back_end_filter=('hp',)):
+def getInteractiveDeviceURI(bus='cups,usb,par', filter=DEFAULT_FILTER, back_end_filter=('hp',)):
     probed_devices = probeDevices(bus.lower(), filter=filter)
     cups_printers = cups.getPrinters()
     log.debug(probed_devices)
@@ -249,7 +274,7 @@ def getInteractiveDeviceURI(bus='cups,usb,par', filter='none', back_end_filter=(
 
 
 def probeDevices(bus='cups,usb,par', timeout=10,
-                 ttl=4, filter='none',  search='', net_search='slp', 
+                 ttl=4, filter=DEFAULT_FILTER,  search='', net_search='slp', 
                  back_end_filter=('hp',)):
 
     num_devices, ret_devices = 0, {}
@@ -312,16 +337,10 @@ def probeDevices(bus='cups,usb,par', timeout=10,
                             elif int(mq.get('support-type', SUPPORT_TYPE_NONE)) == SUPPORT_TYPE_NONE:
                                 log.debug("Not supported.")
                                 include = False
-
-                            elif filter not in ('none', 'print'):
-                                for f in filter.split(','):
-                                    filter_type = int(mq.get('%s-type' % f.lower().strip(), 0))
-
-                                    if filter_type == 0:
-                                        log.debug("%s filtered out." % device_uri)
-                                        include = False
-                                        break
-
+                            
+                            elif filter not in (None, 'print', 'print-type'):
+                                include = __checkFilter(filter, mq)
+                                
                             if include:
                                 ret_devices[device_uri] = (model, model, hn)
 
@@ -363,15 +382,8 @@ def probeDevices(bus='cups,usb,par', timeout=10,
                             log.debug("Not supported.")
                             include = False
 
-                        elif filter not in ('none', 'print'):
-
-                            for f in filter.split(','):
-                                filter_type = int(mq.get('%s-type' % f.lower().strip(), 0))
-
-                                if filter_type == 0:
-                                    log.debug("%s filtered out." % uri)
-                                    include = False
-                                    break
+                        elif filter not in (None, 'print', 'print-type'):
+                            include = __checkFilter(filter, mq)
 
                         if include:
                             ret_devices[uri] = (mdl, desc, devid) # model w/ _'s, mdl w/o
@@ -389,7 +401,7 @@ def probeDevices(bus='cups,usb,par', timeout=10,
                         back_end, is_hp, bs, model, serial, dev_file, host, port = \
                             parseDeviceURI(device_uri)
                     except Error:
-                        log.warning("Inrecognized URI: %s" % device_uri)
+                        log.debug("Unrecognized URI: %s" % device_uri)
                         continue
 
                     if not is_hp:
@@ -406,14 +418,8 @@ def probeDevices(bus='cups,usb,par', timeout=10,
                         log.debug("Not supported.")
                         include = False
 
-                    elif filter not in ('none', 'print'):
-                        for f in filter.split(','):
-                            filter_type = int(mq.get('%s-type' % f.lower().strip(), 0))
-
-                            if filter_type == 0:
-                                log.debug("%s filtered out." % device_uri)
-                                include = False
-                                break
+                    elif filter not in (None, 'print', 'print-type'):
+                        include = __checkFilter(filter, mq)
 
                     if include:
                         ret_devices[device_uri] = (model, model, '')
@@ -434,6 +440,7 @@ def probeDevices(bus='cups,usb,par', timeout=10,
         if include:
             probed_devices[uri] = ret_devices[uri]
 
+    cleanup_spinner()
     return probed_devices
 
 
@@ -449,7 +456,7 @@ def getSupportedCUPSDevices(back_end_filter=['hp']):
         except Error:
             continue
 
-        if back_end in back_end_filter:
+        if back_end_filter == '*' or back_end in back_end_filter:
             try:
                 devices[p.device_uri]
             except KeyError:
@@ -509,6 +516,8 @@ def parseDynamicCounter(ctr_field, convert_to_int=True):
 
 
 def parseDeviceURI(device_uri):
+    #print repr(pat_deviceuri)
+    #print repr(device_uri)
     m = pat_deviceuri.match(device_uri)
 
     if m is None:
@@ -553,6 +562,9 @@ def validateBusList(bus, allow_cups=True):
     return True
 
 def validateFilterList(filter):
+    if filter is None:
+        return True
+        
     for f in filter.split(','):
         if f not in VALID_FILTERS:
             log.error("Invalid term '%s' in filter list" % f)
@@ -618,6 +630,7 @@ def queryString(string_id, typ=0):
     if not strings_init:
         initStrings()
 
+    #log.debug("queryString(%s)" % string_id)
     s = st.string_table.get(str(string_id), ('', ''))[typ]
 
     if type(s) == type(''):
@@ -666,7 +679,7 @@ AGENT_kinds = {AGENT_KIND_NONE            : 'invalid',
 
 AGENT_healths = {AGENT_HEALTH_OK           : 'ok',
                   AGENT_HEALTH_MISINSTALLED : 'misinstalled', # supply/cart
-                  AGENT_HEALTH_FAIR_MODERATE : '',
+                  #AGENT_HEALTH_FAIR_MODERATE : '',
                   AGENT_HEALTH_INCORRECT    : 'incorrect',
                   AGENT_HEALTH_FAILED       : 'failed',
                   AGENT_HEALTH_OVERTEMP     : 'overtemp', # battery
@@ -1012,7 +1025,12 @@ class Device(object):
     def closeCfgDownload(self):
         return self.__closeChannel(hpmudext.HPMUD_S_CONFIG_DOWNLOAD_CHANNEL)
 
-
+    def openSoapFax(self):
+        return self.__openChannel(hpmudext.HPMUD_S_SOAP_FAX)
+        
+    def closeSoapFax(self):
+        return self.__closeChannel(hpmudext.HPMUD_S_SOAP_FAX)
+        
     def __closeChannel(self, service_name):
         #if not self.mq['io-mode'] == IO_MODE_UNI and \
         if self.io_state == IO_STATE_HP_OPEN:
@@ -1182,8 +1200,8 @@ class Device(object):
         status_type = self.mq.get('status-type', STATUS_TYPE_NONE)
         battery_check = self.mq.get('status-battery-check', STATUS_BATTERY_CHECK_NONE)
         dynamic_counters = self.mq.get('status-dynamic-counters', STATUS_DYNAMIC_COUNTERS_NONE)
-        #io_mode = self.mq.get('io-mode', IO_MODE_UNI)
-        #io_mfp_mode = self.mq.get('io-mfp-mode', IO_MODE_UNI)
+        io_mode = self.mq.get('io-mode', IO_MODE_UNI)
+        io_mfp_mode = self.mq.get('io-mfp-mode', IO_MODE_UNI)
 
         # Turn off status if local connection and bi-di not avail.
         #if io_mode  == IO_MODE_UNI and self.back_end != 'net':
@@ -1224,20 +1242,26 @@ class Device(object):
                 log.debug("Type 1/2 (S: or VSTATUS:) status")
                 status_block = status.parseStatus(self.deviceID)
 
-            elif status_type == STATUS_TYPE_LJ:
-                log.debug("Type 3 LaserJet status")
+            elif status_type in (STATUS_TYPE_LJ, STATUS_TYPE_PML_AND_PJL):
+                log.debug("Type 3/9 LaserJet PML(+PJL) status")
                 status_block = status.StatusType3(self, self.deviceID)
 
             elif status_type == STATUS_TYPE_LJ_XML:
                 log.debug("Type 6: LJ XML")
                 status_block = status.StatusType6(self)
+                
+            elif status_type == STATUS_TYPE_PJL:
+                log.debug("Type 8: LJ PJL")
+                status_block = status.StatusType8(self)
 
             else:
                 log.error("Unimplemented status type: %d" % status_type)
 
-            if battery_check:
+            if battery_check and \
+                io_mode != IO_MODE_UNI:
+                
                 log.debug("Battery check...")
-                status.BatteryCheck(self, status_block)
+                status.BatteryCheck(self, status_block, battery_check)
 
             if status_block:
                 log.debug(status_block)
@@ -1255,7 +1279,8 @@ class Device(object):
 
             if not quick and \
                 self.mq.get('fax-type', FAX_TYPE_NONE) and \
-                status_code == STATUS_PRINTER_IDLE:
+                status_code == STATUS_PRINTER_IDLE and \
+                io_mode != IO_MODE_UNI:
 
                 log.debug("Fax activity check...")
 
@@ -1291,7 +1316,10 @@ class Device(object):
                 if self.panel_check:
                     self.panel_check = bool(self.mq.get('panel-check-type', 0))
 
-                if self.panel_check and status_type in (STATUS_TYPE_LJ, STATUS_TYPE_S, STATUS_TYPE_VSTATUS):
+                if self.panel_check and \
+                    status_type in (STATUS_TYPE_LJ, STATUS_TYPE_S, STATUS_TYPE_VSTATUS) and \
+                    io_mode != IO_MODE_UNI:
+                    
                     log.debug("Panel check...")
                     try:
                         self.panel_check, line1, line2 = status.PanelCheck(self)
@@ -1303,7 +1331,9 @@ class Device(object):
                                       'panel-line2': line2,})
 
 
-                if dynamic_counters != STATUS_DYNAMIC_COUNTERS_NONE:
+                if dynamic_counters != STATUS_DYNAMIC_COUNTERS_NONE and \
+                    io_mode != IO_MODE_UNI:
+                    
                     r_value, r_value_str, rg, rr = self.getRValues(r_type, status_type, dynamic_counters)
                 else:
                     r_value, r_value_str, rg, rr = 0, '000000000', '000', '000000'
@@ -1346,9 +1376,9 @@ class Device(object):
 
                 a = 1
                 while True:
-                    mq_agent_kind = self.mq.get('r%d-agent%d-kind' % (r_value, a), 0)
+                    mq_agent_kind = self.mq.get('r%d-agent%d-kind' % (r_value, a), -1)
 
-                    if mq_agent_kind == 0:
+                    if mq_agent_kind == -1:
                         break
 
                     mq_agent_type = self.mq.get('r%d-agent%d-type' % (r_value, a), 0)
@@ -1366,13 +1396,16 @@ class Device(object):
                            break
 
                     if found:
-                        log.debug("r%d-kind%d-type%d" % (r_value, agent_kind, agent_type))
+                        log.debug("found: r%d-kind%d-type%d" % (r_value, agent_kind, agent_type))
 
-                        log.debug("found")
                         agent_health = agent.get('health', AGENT_HEALTH_OK)
+                        agent_level = agent.get('level', 100)
                         agent_level_trigger = agent.get('level-trigger',
                             AGENT_LEVEL_TRIGGER_SUFFICIENT_0)
 
+                        log.debug("health=%d, level=%d, level_trigger=%d, status_code=%d" % 
+                            (agent_health, agent_level, agent_level_trigger, status_code))
+                        
                         query = 'agent_%s_%s' % (AGENT_types.get(agent_type, 'unknown'), 
                                                  AGENT_kinds.get(agent_kind, 'unknown'))
 
@@ -1383,12 +1416,11 @@ class Device(object):
                         # if agent health is OK, check for low supplies. If low, use
                         # the agent level trigger description for the agent description.
                         # Otherwise, report the agent health.
-                        if status_code == STATUS_PRINTER_IDLE and \
+                        if (status_code == STATUS_PRINTER_IDLE or status_code == STATUS_PRINTER_OUT_OF_INK) and \
                             (agent_health == AGENT_HEALTH_OK or 
                              (agent_health == AGENT_HEALTH_FAIR_MODERATE and agent_kind == AGENT_KIND_HEAD)) and \
                             agent_level_trigger >= AGENT_LEVEL_TRIGGER_MAY_BE_LOW:
 
-                            # Low
                             query = 'agent_level_%s' % AGENT_levels.get(agent_level_trigger, 'unknown')
 
                             if tech_type in (TECH_TYPE_MONO_INK, TECH_TYPE_COLOR_INK):
@@ -1402,105 +1434,101 @@ class Device(object):
                             self.dq['error-state'] = STATUS_TO_ERROR_STATE_MAP.get(code, ERROR_STATE_LOW_SUPPLIES)
                             self.sendEvent(code)
 
-                            if agent_level_trigger in (AGENT_LEVEL_TRIGGER_PROBABLY_OUT, AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT):
+                            if agent_level_trigger in \
+                                (AGENT_LEVEL_TRIGGER_PROBABLY_OUT, AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT):
+                                
                                 query = 'agent_level_out'
                             else:
                                 query = 'agent_level_low'
 
-                        agent_health_desc = self.queryString(query)
-
-                        self.dq.update(
-                        {
-                            'agent%d-kind' % a :          agent_kind,
-                            'agent%d-type' % a :          agent_type,
-                            'agent%d-known' % a :         agent.get('known', False),
-                            'agent%d-sku' % a :           mq_agent_sku,
-                            'agent%d-level' % a :         agent.get('level', 0),
-                            'agent%d-level-trigger' % a : agent_level_trigger,
-                            'agent%d-ack' % a :           agent.get('ack', False),
-                            'agent%d-hp-ink' % a :        agent.get('hp-ink', False),
-                            'agent%d-health' % a :        agent_health,
-                            'agent%d-dvc' % a :           agent.get('dvc', 0),
-                            'agent%d-virgin' % a :        agent.get('virgin', False),
-                            'agent%d-desc' % a :          agent_desc,
-                            'agent%d-id' % a :            agent.get('id', 0),
-                            'agent%d-health-desc' % a :   agent_health_desc,
-                        })
-
-                    else:
-                        agent_health = AGENT_HEALTH_MISINSTALLED
-                        agent_level_trigger = AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT
-
-                        query = 'agent_%s_%s' % (AGENT_types.get(mq_agent_type, 'unknown'),
-                                                 AGENT_kinds.get(mq_agent_kind, 'unknown'))
-
-                        agent_desc = self.queryString(query)
-                        agent_health_desc = self.queryString("agent_health_misinstalled")
-
-                        self.dq.update(
-                        {
-                            'agent%d-kind' % a :          mq_agent_kind,
-                            'agent%d-type' % a :          mq_agent_type,
-                            'agent%d-known' % a :         False,
-                            'agent%d-sku' % a :           mq_agent_sku,
-                            'agent%d-level' % a :         0,
-                            'agent%d-level-trigger' % a : agent_level_trigger,
-                            'agent%d-ack' % a :           False,
-                            'agent%d-hp-ink' % a :        False,
-                            'agent%d-health' % a :        agent_health,
-                            'agent%d-dvc' % a :           0,
-                            'agent%d-virgin' % a :        False,
-                            'agent%d-desc' % a :          agent_desc,
-                            'agent%d-id' % a :            0,
-                            'agent%d-health-desc' % a :   agent_health_desc,
-                        })
+                            agent_health_desc = self.queryString(query)
+    
+                            self.dq.update(
+                            {
+                                'agent%d-kind' % a :          agent_kind,
+                                'agent%d-type' % a :          agent_type,
+                                'agent%d-known' % a :         agent.get('known', False),
+                                'agent%d-sku' % a :           mq_agent_sku,
+                                'agent%d-level' % a :         agent_level,
+                                'agent%d-level-trigger' % a : agent_level_trigger,
+                                'agent%d-ack' % a :           agent.get('ack', False),
+                                'agent%d-hp-ink' % a :        agent.get('hp-ink', False),
+                                'agent%d-health' % a :        agent_health,
+                                'agent%d-dvc' % a :           agent.get('dvc', 0),
+                                'agent%d-virgin' % a :        agent.get('virgin', False),
+                                'agent%d-desc' % a :          agent_desc,
+                                'agent%d-id' % a :            agent.get('id', 0),
+                                'agent%d-health-desc' % a :   agent_health_desc,
+                            })
 
+                        else:
+                            query = 'agent_health_%s' % AGENT_healths.get(agent_health, AGENT_HEALTH_OK)
+                            agent_health_desc = self.queryString(query)
+    
+                            self.dq.update(
+                            {
+                                'agent%d-kind' % a :          agent_kind,
+                                'agent%d-type' % a :          agent_type,
+                                'agent%d-known' % a :         False,
+                                'agent%d-sku' % a :           mq_agent_sku,
+                                'agent%d-level' % a :         agent_level,
+                                'agent%d-level-trigger' % a : agent_level_trigger,
+                                'agent%d-ack' % a :           False,
+                                'agent%d-hp-ink' % a :        False,
+                                'agent%d-health' % a :        agent_health,
+                                'agent%d-dvc' % a :           0,
+                                'agent%d-virgin' % a :        False,
+                                'agent%d-desc' % a :          agent_desc,
+                                'agent%d-id' % a :            0,
+                                'agent%d-health-desc' % a :   agent_health_desc,
+                            })
+    
                     a += 1
 
-                else: # Create agent keys for not-found devices
+        else: # Create agent keys for not-found devices
 
-                    r_value = 0
-                    if r_type > 0 and self.r_values is not None:
-                        r_value = self.r_values[0]
+            r_value = 0
+            if r_type > 0 and self.r_values is not None:
+                r_value = self.r_values[0]
 
-                    # Make sure there is some valid agent data for this r_value
-                    # If not, fall back to r_value == 0
-                    if r_value > 0 and self.mq.get('r%d-agent1-kind', 0) == 0:
-                        r_value = 0
+            # Make sure there is some valid agent data for this r_value
+            # If not, fall back to r_value == 0
+            if r_value > 0 and self.mq.get('r%d-agent1-kind', 0) == 0:
+                r_value = 0
 
-                    a = 1
-                    while True:
-                        mq_agent_kind = self.mq.get('r%d-agent%d-kind' % (r_value, a), 0)
+            a = 1
+            while True:
+                mq_agent_kind = self.mq.get('r%d-agent%d-kind' % (r_value, a), 0)
 
-                        if mq_agent_kind == 0:
-                            break
+                if mq_agent_kind == 0:
+                    break
 
-                        mq_agent_type = self.mq.get('r%d-agent%d-type' % (r_value, a), 0)
-                        mq_agent_sku = self.mq.get('r%d-agent%d-sku' % (r_value, a), '')
-                        query = 'agent_%s_%s' % (AGENT_types.get(mq_agent_type, 'unknown'),
-                                                 AGENT_kinds.get(mq_agent_kind, 'unknown'))
+                mq_agent_type = self.mq.get('r%d-agent%d-type' % (r_value, a), 0)
+                mq_agent_sku = self.mq.get('r%d-agent%d-sku' % (r_value, a), '')
+                query = 'agent_%s_%s' % (AGENT_types.get(mq_agent_type, 'unknown'),
+                                         AGENT_kinds.get(mq_agent_kind, 'unknown'))
 
-                        agent_desc = self.queryString(query)
+                agent_desc = self.queryString(query)
 
-                        self.dq.update(
-                        {
-                            'agent%d-kind' % a :          mq_agent_kind,
-                            'agent%d-type' % a :          mq_agent_type,
-                            'agent%d-known' % a :         False,
-                            'agent%d-sku' % a :           mq_agent_sku,
-                            'agent%d-level' % a :         0,
-                            'agent%d-level-trigger' % a : AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT,
-                            'agent%d-ack' % a :           False,
-                            'agent%d-hp-ink' % a :        False,
-                            'agent%d-health' % a :        AGENT_HEALTH_MISINSTALLED,
-                            'agent%d-dvc' % a :           0,
-                            'agent%d-virgin' % a :        False,
-                            'agent%d-health-desc' % a :   self.queryString('agent_health_unknown'),
-                            'agent%d-desc' % a :          agent_desc,
-                            'agent%d-id' % a :            0,
-                        })
+                self.dq.update(
+                {
+                    'agent%d-kind' % a :          mq_agent_kind,
+                    'agent%d-type' % a :          mq_agent_type,
+                    'agent%d-known' % a :         False,
+                    'agent%d-sku' % a :           mq_agent_sku,
+                    'agent%d-level' % a :         0,
+                    'agent%d-level-trigger' % a : AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT,
+                    'agent%d-ack' % a :           False,
+                    'agent%d-hp-ink' % a :        False,
+                    'agent%d-health' % a :        AGENT_HEALTH_MISINSTALLED,
+                    'agent%d-dvc' % a :           0,
+                    'agent%d-virgin' % a :        False,
+                    'agent%d-health-desc' % a :   self.queryString('agent_health_unknown'),
+                    'agent%d-desc' % a :          agent_desc,
+                    'agent%d-id' % a :            0,
+                })
 
-                        a += 1
+                a += 1
 
         for d in self.dq:
             self.__dict__[d.replace('-','_')] = self.dq[d]
@@ -1523,28 +1551,49 @@ class Device(object):
         result_code, data, typ, pml_result_code = \
             hpmudext.get_pml(self.device_id, channel_id, pml.PMLToSNMP(oid[0]), oid[1])
 
-        log.debug("Result code = 0x%x" % pml_result_code)
-
-        if pml_result_code >= pml.ERROR_UNKNOWN_REQUEST:
+        if pml_result_code > pml.ERROR_MAX_OK:
+            log.debug("PML/SNMP GET %s failed (result code = 0x%x)" % (oid[0], pml_result_code))
             return pml_result_code, None
 
-        return pml_result_code, pml.ConvertFromPMLDataFormat(data, oid[1], desired_int_size)
+        converted_data = pml.ConvertFromPMLDataFormat(data, oid[1], desired_int_size)
+        
+        if log.is_debug():
+            if oid[1] in (pml.TYPE_STRING, pml.TYPE_BINARY):
+                
+                log.debug("PML/SNMP GET %s (result code = 0x%x) returned:" % 
+                    (oid[0], pml_result_code))
+                log.log_data(data)
+            else:
+                log.debug("PML/SNMP GET %s (result code = 0x%x) returned: %s" % 
+                    (oid[0], pml_result_code, repr(converted_data)))
+        
+        return pml_result_code, converted_data
 
 
     def setPML(self, oid, value): # oid => ( 'dotted oid value', pml type )
         channel_id = self.openPML()
 
         value = pml.ConvertToPMLDataFormat(value, oid[1])
-
+        
         result_code, pml_result_code = \
             hpmudext.set_pml(self.device_id, channel_id, pml.PMLToSNMP(oid[0]), oid[1], value)
-
-        log.debug("Result code = 0x%x" % pml_result_code)
+            
+        if log.is_debug():
+            if oid[1] in (pml.TYPE_STRING, pml.TYPE_BINARY): 
+                
+                log.debug("PML/SNMP SET %s (result code = 0x%x) to:" % 
+                    (oid[0], pml_result_code))
+                log.log_data(value)
+            else:
+                log.debug("PML/SNMP SET %s (result code = 0x%x) to: %s" % 
+                    (oid[0], pml_result_code, repr(value)))
+        
         return pml_result_code
 
 
     def getDynamicCounter(self, counter, convert_to_int=True):
         dynamic_counters = self.mq.get('status-dynamic-counters', STATUS_DYNAMIC_COUNTERS_NONE)
+        log.debug("Dynamic counters: %d" % dynamic_counters)
         if dynamic_counters != STATUS_DYNAMIC_COUNTERS_NONE:
 
             if dynamic_counters == STATUS_DYNAMIC_COUNTERS_LIDIL_0_5_4:
@@ -1598,21 +1647,23 @@ class Device(object):
             raise Error(ERROR_DEVICE_DOES_NOT_SUPPORT_OPERATION)
 
 
-    def readPrint(self, bytes_to_read, stream=None, timeout=prop.read_timeout):
-        return self.__readChannel(self.openPrint, bytes_to_read, stream, timeout)
+    def readPrint(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=False):
+        return self.__readChannel(self.openPrint, bytes_to_read, stream, timeout, allow_short_read)
 
-    def readPCard(self, bytes_to_read, stream=None, timeout=prop.read_timeout):
-        return self.__readChannel(self.openPCard, bytes_to_read, stream, timeout)
+    def readPCard(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=False):
+        return self.__readChannel(self.openPCard, bytes_to_read, stream, timeout, allow_short_read)
 
-    def readFax(self, bytes_to_read, stream=None, timeout=prop.read_timeout):
-        return self.__readChannel(self.openFax, bytes_to_read, stream, timeout)
+    def readFax(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=False):
+        return self.__readChannel(self.openFax, bytes_to_read, stream, timeout, allow_short_read)
 
-    def readCfgUpload(self, bytes_to_read, stream=None, timeout=prop.read_timeout):
-        return self.__readChannel(self.openCfgUpload, bytes_to_read, stream, timeout)
+    def readCfgUpload(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=False):
+        return self.__readChannel(self.openCfgUpload, bytes_to_read, stream, timeout, allow_short_read)
 
-    def readEWS(self, bytes_to_read, stream=None, timeout=prop.read_timeout):
-        return self.__readChannel(self.openEWS, bytes_to_read, stream, timeout, True)
+    def readEWS(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
+        return self.__readChannel(self.openEWS, bytes_to_read, stream, timeout, allow_short_read)
 
+    def readSoapFax(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
+        return self.__readChannel(self.openSoapFax, bytes_to_read, stream, timeout, allow_short_read)
 
     def __readChannel(self, opener, bytes_to_read, stream=None, 
                       timeout=prop.read_timeout, allow_short_read=False):
@@ -1627,14 +1678,14 @@ class Device(object):
             buffer = ''
 
         while True:
-            #print self.device_id, channel_id, bytes_to_read, timeout
             result_code, data = \
                 hpmudext.read_channel(self.device_id, channel_id, bytes_to_read, timeout)
 
-            #result_code, data = read_channel(dd, cd, bytes_to_read, [timeout])
-
             l = len(data)
-            #log.log_data(data)
+            
+            if result_code == hpmudext.HPMUD_R_IO_TIMEOUT:
+                log.debug("I/O timeout")
+                break
 
             if result_code != hpmudext.HPMUD_R_OK: 
                 log.error("Channel read error")
@@ -1654,15 +1705,19 @@ class Device(object):
             if self.callback is not None:
                 self.callback()
 
-            if num_bytes == bytes_to_read or allow_short_read:
-                log.debug("Read complete")
+            if num_bytes == bytes_to_read:
+                log.debug("Full read complete.")
+                break
+                
+            if allow_short_read and num_bytes < bytes_to_read:
+                log.debug("Allowed short read of %d of %d bytes complete." % (num_bytes, bytes_to_read))
                 break
 
         if stream is None:
             log.debug("Returned %d total bytes in buffer." % num_bytes)
             return buffer
         else:
-            log.debug("Wrote %d total bytes to stream." % num_bytes)
+            log.debug("Saved %d total bytes to stream." % num_bytes)
             return num_bytes
 
 
@@ -1680,6 +1735,10 @@ class Device(object):
 
     def writeCfgDownload(self, data):
         return self.__writeChannel(self.openCfgDownload, data)
+        
+    def writeSoapFax(self, data):
+        return self.__writeChannel(self.openSoapFax, data)
+    
 
     def __writeChannel(self, opener, data):
         channel_id = opener()
@@ -1719,6 +1778,8 @@ class Device(object):
                                                     value,
                                                     oid[1])))
 
+        log.log_data(data)
+        
         self.printData(data, direct=direct, raw=True)
 
 
@@ -1813,7 +1874,7 @@ class Device(object):
 
             else:
                 raw_str, rem_str = '', ''
-                if raw: raw_str = '-l'
+                if raw: raw_str = '-o raw'
                 if remove: rem_str = '-r'
 
                 if is_gzip:
@@ -1835,6 +1896,7 @@ class Device(object):
 
     def printData(self, data, printer_name=None, direct=True, raw=True):
         #log.log_data(data)
+        #log.debug("printData(direct=%s, raw=%s)" % (direct, raw))
         if direct:
             self.writePrint(data)
         else:
@@ -1887,7 +1949,7 @@ class Device(object):
     def getEWSUrl(self, url, stream):
         try:
             if self.is_local:
-                url2 = "%s&loc=%s" % (self.device_uri, url)
+                url2 = "%s&loc=%s" % (self.device_uri.replace('hpfax:', 'hp:'), url)
                 data = self
             else:
                 url2 = "http://%s%s" % (self.host, url)
@@ -1911,21 +1973,40 @@ class Device(object):
             self.closeEWS()
 
 
-    def downloadFirmware(self):
+    def downloadFirmware(self, usb_bus_id=None, usb_device_id=None):
         ok = False
-        filename = os.path.join(prop.data_dir, "firmware", self.model + '.fw.gz')
+        filename = os.path.join(prop.data_dir, "firmware", self.model.lower() + '.fw.gz')
         log.debug(filename)
 
         if os.path.exists(filename):
-            try:
-                log.debug("Downloading firmware file '%s'..." % filename)
-                self.openPrint()
-                bytes_written = self.writePrint(gzip.open(filename).read())
-                log.debug("%s bytes downloaded." % utils.commafy(bytes_written))
-                self.closePrint()
-                ok = True
-            except:
-                log.error("An error occured.")
+            log.debug("Downloading firmware file '%s'..." % filename)
+            
+            # Write to port directly (no MUD) so that HAL can enumerate the printer
+            if 0: # this currently doesn't work because usblp is loaded...
+            #if usb_bus_id is not None and usb_device_id is not None:
+                try:
+                    p = "/dev/bus/usb/%s/%s" % (usb_bus_id, usb_device_id)
+                    log.debug("Writing to %s..." % p)
+                    #f = file(p, 'w')
+                    f = os.open(p, os.O_RDWR)
+                    x = gzip.open(filename).read()
+                    os.write(f, x)
+                    #f.close()
+                    os.close(f)
+                    ok = True
+                    log.debug("OK")
+                except (OSError, IOError), e:
+                    log.error("An error occured: %s" % e)
+            else:
+                try:
+                    self.openPrint()
+                    bytes_written = self.writePrint(gzip.open(filename).read())
+                    log.debug("%s bytes downloaded." % utils.commafy(bytes_written))
+                    self.closePrint()
+                    ok = True
+                    log.debug("OK")
+                except Error, e:
+                    log.error("An error occured: %s" % e.msg)
         else:
             log.error("Firmware file '%s' not found." % filename)
 
@@ -1955,9 +2036,12 @@ class LocalOpener(urllib.URLopener):
         dev.writeEWS("""GET %s HTTP/1.0\nContent-Length:0\nHost:localhost\nUser-Agent:hplip\n\n""" % loc)
 
         reply = xStringIO()
-        dev.readEWS(MAX_BUFFER, reply)
-
+        
+        while dev.readEWS(8192, reply, timeout=1):
+            pass
+            
         reply.seek(0)
+        log.log_data(reply.getvalue())
 
         response = httplib.HTTPResponse(reply)
         response.begin()
