@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2006 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@ import sys, struct
 from g import *
 from base.utils import unprintable
 
-
 # Request codes
 GET_REQUEST = 0x00
 GET_NEXT_REQUEST = 0x01
@@ -289,6 +288,7 @@ def parsePMLPacket(p, expected_data_type=TYPE_UNKNOWN):
     return data, data_type, error_code
 
 
+
 def HPToSNMP(oid): # 1.
     return '.'.join(['1.3.6.1.4.1.11.2.3.9.4.2', oid, '0'])
 
@@ -319,6 +319,9 @@ def PMLToSNMP(oid):
 #
 # OIDs
 #
+
+OID_DEVICE_ID = ('0.1.3.6.1.4.1.11.2.3.9.1.1.7.0', TYPE_STRING) # network/SNMP only (undocumented)
+
 OID_DEVICE_SUPPORTED_FUNCTIONS = ('1.1.2.67', TYPE_COLLECTION)
 DEVICE_SUPPORTED_FUNCTIONS_SCAN =                 0x00002
 DEVICE_SUPPORTED_FUNCTIONS_SCAN_SIMPLEX =         0x00004
@@ -353,8 +356,10 @@ PRINT_INTERNAL_PAGE_SUPPLIES_PAGE = 101
 PRINT_INTERNAL_PAGE_COLOR_PALETTE_CMYK_PAGE = 259
 PRINT_INTERNAL_PAGE_COLOR_CAL = 1102
 PRINT_INTERNAL_PAGE_COLOR_CAL_VERIFICATION = 1104
-PRINT_INTERNAL_PAGE_ALIGNMENT_PAGE = 1100 # LBOW
+PRINT_INTERNAL_PAGE_ALIGNMENT_PAGE = 1100 # LBOW/OJ Pro L7xxx
 PRINT_INTERNAL_PAGE_ALIGNMENT_PAGE_VERIFICATION = 1150 # LBOW
+PRINT_INTERNAL_PAGE_LINEFEED_CALIBRATION = 1407 # OJ Pro
+PRINT_INTERNAL_PAGE_PRINT_QUALITY_DIAGNOSTIC = 1409 # OJ Pro
 
 # From xojpanel
 OID_SPM_LINE1 = ('2.16.5.1.2.1.1', TYPE_STRING)
@@ -505,6 +510,15 @@ POWER_MODE_CHARGING = 0x04
 POWER_MODE_DISCHARGING = 0x08
 POWER_MODE_BATTERY_LEVEL_KNOWN = 0x10
 
+OID_POWER_SETTINGS = ('1.1.2.118', TYPE_ENUMERATION)
+OID_POWER_SETTINGS_15MIN = 1
+OID_POWER_SETTINGS_30MIN = 2
+OID_POWER_SETTINGS_45MIN = 3
+OID_POWER_SETTINGS_1HR = 4
+OID_POWER_SETTINGS_2HR = 5
+OID_POWER_SETTINGS_3HR = 6
+OID_POWER_SETTINGS_NEVER = 999
+
 #
 # Fax
 #
@@ -689,6 +703,31 @@ OID_COPIER_CONTRAST_AIO = ('1.5.1.2', TYPE_SIGNED_INTEGER)
 OID_COPIER_REDUCTION_AIO = ('1.5.1.4', TYPE_SIGNED_INTEGER)
 OID_COPIER_QUALITY_AIO = ('1.5.1.13', TYPE_ENUMERATION)
 
+OID_COPIER_JOB_INPUT_TRAY_SELECT = ('1.5.1.51', TYPE_SIGNED_INTEGER)
+COPIER_JOB_INPUT_TRAY_1 = 1
+COPIER_JOB_INPUT_TRAY_2 = 1
+COPIER_JOB_INPUT_TRAY_3 = 1
+
+OID_COPIER_MEDIA_TYPE = ('1.5.1.14', TYPE_SIGNED_INTEGER)
+COPIER_MEDIA_TYPE_PLAIN               = 1
+COPIER_MEDIA_TYPE_BRIGHT_WHITE        = 2
+COPIER_MEDIA_TYPE_PREMIUM_PHOTO       = 3
+COPIER_MEDIA_TYPE_SPECIAL             = 4
+COPIER_MEDIA_TYPE_TRANSPARENCY        = 5
+COPIER_MEDIA_TYPE_IRON_ON             = 6
+COPIER_MEDIA_TYPE_FAST_TRANSPARANCEY  = 7
+COPIER_MEDIA_TYPE_BROCHURE_MATTE      = 8
+COPIER_MEDIA_TYPE_BROCHURE_GLOSSY     = 9
+COPIER_MEDIA_TYPE_PHOTO_GLOSSY        = 10
+COPIER_MEDIA_TYPE_MATTE_PAPER         = 11
+COPIER_MEDIA_TYPE_EVERYDAY_PHOTO      = 12
+COPIER_MEDIA_TYPE_PHOTO_QUAL_INKJET   = 13
+COPIER_MEDIA_TYPE_PHOTO               = 14 
+COPIER_MEDIA_TYPE_AUTOMATIC           = 15
+COPIER_MEDIA_TYPE_ADVANCED_PHOTO      = 16
+COPIER_MEDIA_TYPE_IRON_ON_MIRRORED    = 17
+
+
 
 # Misc
 
