@@ -89,7 +89,8 @@ DJ9xxVIP::DJ9xxVIP
     pMode[ModeCount++] = new DJ9902400Mode ();         // HiRes
     pMode[ModeCount++] = new DJ990DraftMode ();        // Draft Color
 #endif
-    pMode[ModeCount++] = new DJ990BestMode ();        // Photo Best
+    pMode[ModeCount++] = new DJ990BestMode ();         // Photo Best
+    pMode[ModeCount++] = new DJ990PhotoNormalMode ();  // Photo Normal
 
 }
 
@@ -267,6 +268,28 @@ DJ990BestMode::DJ990BestMode () : PrintMode (NULL)
     pmMediaType = MEDIA_PHOTO;
 } // DJ990BestMode
 
+DJ990PhotoNormalMode::DJ990PhotoNormalMode () : PrintMode (NULL)
+{
+    BaseResX = BaseResY = ResolutionX[0] = ResolutionY[0] = VIP_BASE_RES;
+
+	bFontCapable = FALSE;
+
+#ifdef  APDK_AUTODUPLEX
+	bDuplexCapable = TRUE;
+#endif
+
+#if defined(APDK_VIP_COLORFILTERING)
+    Config.bErnie = TRUE;
+#endif
+
+   Config.bColorImage = FALSE;
+
+	medium      = mediaGlossy;
+	theQuality  = qualityNormal;
+	pmQuality   = QUALITY_NORMAL;
+    pmMediaType = MEDIA_PHOTO;
+} // DJ990PhotoNormalMode
+
 BOOL DJ9xxVIP::UseGUIMode
 (
     PrintMode* pPrintMode
@@ -531,6 +554,30 @@ DRIVER_ERROR HeaderDJ990::Send()
     ERRCHECK;
 
     ////////////////////////////////////////////////////////////////////////////////////
+
+/*
+ *  Custom papersize command
+ */
+
+    if (thePrintContext->thePaperSize == CUSTOM_SIZE)
+    {
+        BYTE    szStr[32];
+        short   sWidth, sHeight;
+        BYTE    b1, b2;
+        sWidth  = (short) (thePrintContext->PhysicalPageSizeX () * thePrintContext->EffectiveResolutionX ());
+        sHeight = (short) (thePrintContext->PhysicalPageSizeY () * thePrintContext->EffectiveResolutionY ());
+        memcpy (szStr, "\x1B*o5W\x0E\x05\x00\x00\x00\x1B*o5W\x0E\x06\x00\x00\x00", 20);
+        b1 = (BYTE) ((sWidth & 0xFF00) >> 8);
+        b2 = (BYTE) (sWidth & 0xFF);
+        szStr[8] = b1;
+        szStr[9] = b2;
+        b1 = (BYTE) ((sHeight & 0xFF00) >> 8);
+        b2 = (BYTE) (sHeight & 0xFF);
+        szStr[18] = b1;
+        szStr[19] = b2;
+        err = thePrinter->Send ((const BYTE *) szStr, 20);
+    }
+
     float   fXOverSpray = 0.0;
     float   fYOverSpray = 0.0;
     float   fLeftOverSpray = 0.0;
@@ -575,6 +622,10 @@ DRIVER_ERROR HeaderDJ990::Send()
         err = thePrinter->Send ((const BYTE *) cBuf, 2);
     }
 
+//  Now send media pre-load command
+    err = thePrinter->Send ((const BYTE *) "\033&l-2H", 6);   // Moved from Modes(), des 3/11/03
+    ERRCHECK;
+
     // no need for compression command, it's in the CRD
 
     err = thePrinter->Send((const BYTE*)grafStart, sizeof(grafStart) );
@@ -626,10 +677,6 @@ DRIVER_ERROR HeaderDJ990::StartSend()
     err = Modes ();            // Set media source, type, size and quality modes.
     ERRCHECK;
 
-//  Now send media pre-load command
-    err = thePrinter->Send ((const BYTE *) "\033&l-2H", 6);   // Moved from Modes(), des 3/11/03
-    ERRCHECK;
-
     if (!thePrinter->UseGUIMode(thePrintContext->CurrentMode))
     {
         err = Margins();          // set margins
