@@ -444,7 +444,6 @@ typedef enum
 {
 	eDelta32,
 	eDeltaPlus = 24,
-	eJBIG,
 	eFX = 18,
 	eRAW = 2
 } CompressionMethod;
@@ -1078,6 +1077,7 @@ BOOL ModeDeltaPlus::Compress (HPUInt8   *outmem,
                                &&
                                (
                                  /* seedrow and run */
+                                 ((col + 2) >= horz_ht_dist) &&
                                  (cur_row[col+2-horz_ht_dist] == cur_row[col+2])
                                  ||
                                  /* seedrow and northeast predict */
@@ -1091,6 +1091,8 @@ BOOL ModeDeltaPlus::Compress (HPUInt8   *outmem,
                          (
                            (cur_row[col] != seedrow[col])
                            &&
+                           ((col + 1) >= horz_ht_dist)
+                           &&
                            (cur_row[col+1-horz_ht_dist] == cur_row[col+1])
                            &&
                            (
@@ -1098,6 +1100,8 @@ BOOL ModeDeltaPlus::Compress (HPUInt8   *outmem,
                              (
                                ((col+2) < row_width)
                                &&
+                               ((col + 2) >= horz_ht_dist)
+                               &&
                                (cur_row[col+2-horz_ht_dist] == cur_row[col+2])
                              )
                              ||
@@ -1205,12 +1209,12 @@ BOOL ModeDeltaPlus::Process
 			{
 				err = (((LJFastRaster*)thePrinter)->phLJFastRaster)->StartPage();
 				((LJFastRaster*)thePrinter)->m_bStartPageNotSent = FALSE;
-                m_lPrinterRasterRow = 0;
+						m_lPrinterRasterRow = 0;
 			}
 
 			m_compressedsize = 2 * inputsize * INDY_STRIP_HEIGHT;
             BOOL bRet = Compress (compressBuf, 
-                                  (uint32_t *) &m_compressedsize,
+                                  &m_compressedsize,
                                   pbyInputImageBuffer,
                                   inputsize,
                                   m_lCurrCDRasterRow,
