@@ -1,7 +1,7 @@
 /*****************************************************************************\
   printer.cpp : Implimentation for the Printer class
 
-  Copyright (c) 1996 - 2001, Hewlett-Packard Co.
+  Copyright (c) 1996 - 2006, Hewlett-Packard Co.
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -74,6 +74,10 @@ Printer::Printer
 
     int i = 0; // counter
 
+#ifdef  APDK_LINUX
+    m_iNumPages = 0;
+#endif
+
     for (i = 0; i < MAX_PRINTMODES; i++)
     {
         pMode[i]=NULL;
@@ -188,21 +192,16 @@ Printer::~Printer()
 
 
 ////////////////////////////////////////////////////////////////////////////
-Compressor* Printer::CreateCompressor
-(
-    unsigned int RasterSize
-)
+Compressor* Printer::CreateCompressor (unsigned int RasterSize)
 {
-    return new Mode9(pSS,RasterSize);   // most printers use mode 9
+    return new Mode9 (pSS, RasterSize);   // most printers use mode 9
 }
 
 ////////////////////////////////////////////////////////////////////////////
-Compressor* Printer::CreateBlackPlaneCompressor
-(
-    unsigned int RasterSize
-)
+Compressor* Printer::CreateBlackPlaneCompressor (unsigned int RasterSize,
+                                                 BOOL bVIPPrinter)
 {
-    return new Mode9(pSS,RasterSize);   // most printers use mode 9
+    return new Mode9 (pSS, RasterSize, bVIPPrinter);   // most printers use mode 9
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -1061,6 +1060,7 @@ PrintMode::PrintMode
 
     cmap.ulMap1 = map1;
     cmap.ulMap2 = map2;
+    cmap.ulMap3 = NULL;
 
     BaseResX = BaseResY = TextRes = 300;
     MixedRes= FALSE;
@@ -1194,7 +1194,7 @@ DRIVER_ERROR Printer::SetPenInfo
         else
         {
             TRACE("Printer::SetPenInfo - Unsupported DeviceID %s.\n", pSS->strDevID);
-            ASSERT (0);  // you must have a printer with a new version that is not supported yet!
+/*            ASSERT (0);  // you must have a printer with a new version that is not supported yet! */
             return BAD_DEVICE_ID;  // - code should never reach this point
         }
     }
