@@ -113,8 +113,9 @@ class ScrollStatusView(ScrollView):
                 desc = self.getTimeDeltaDesc(hist[:9])
             else:
                 desc = ''
-        
-        tt = QString("<b>%1 %2</b>").arg(time.strftime("%x %H:%M:%S", hist[:9])).arg(desc).stripWhiteSpace()
+
+        # TODO: In Qt4.x, use QLocale.toString(date, format)
+        tt = QString("<b>%1 %2</b>").arg(QDateTime (QDate(yr, mt, dy), QTime(hr, mi, sec)).toString()).arg(desc).stripWhiteSpace()
         
         self.addGroupHeading(unicode(tt), tt)
         
@@ -215,7 +216,6 @@ class ScrollStatusView(ScrollView):
         self.addWidget(widget, str(self.row))
 
 
-
     def getTimeDeltaDesc(self, past):
         delta = datetime.datetime(*time.localtime()[:7]) - datetime.datetime(*past[:7])
         return self.__tr("(about %1 ago)").arg(self.stringify(delta))
